#
# rclib to abstract details about udhcpc to the dhcp-related scripts.
#
# (C) Copyright 2005-2007, Axis Communications AB, LUND, SWEDEN
# This file is released under the GPL v2.

RC_GOT_DHCP=y

# environment
DHCPC_SCRIPTDIR="/var/run/udhcp.script"
DHCP_RUNDIR="/var/run/dhcp"
DHCP4_RESOLV_CONF="$DHCP_RUNDIR/dhcp4_resolv.conf"
DHCP4_NTP_CONF="$DHCP_RUNDIR/dhcp4_ntp.conf"
# DHCP4_CMD will be evaluated at run time
DHCP4_CMD="/sbin/udhcpc -b -i \$_dhcp_iface -H \$(hostname) -p \$_dhcp_pidf"

# start a dhcp client with the supplied option, for the supplied interface
start_dhcp() {
	[ $# -lt 1 ] && end 1 "$0: missing args"

	_dhcp_iface="$1"
	_dhcp_pidf="$DHCP_RUNDIR/dhcp4_$_dhcp_iface.pid"

	_dhcpc_cmd=$(eval echo $DHCP4_CMD)

	information "starting DHCPv4 client for $_dhcp_iface"
	if check_cmd $_dhcpc_cmd; then
		end 1 "$0: DHCP client already running!"
	else
		$_dhcpc_cmd &
		respawn_on $_dhcpc_cmd || end $?
	fi
}

# stop dhcp clients for the supplied interfaces
stop_dhcp() {
	[ $# -lt 1 ] && end 1 "$0: missing args"

	for _dhcp_iface in $*; do
		_dhcp_pidf="$DHCP_RUNDIR/dhcp4_$_dhcp_iface.pid"
		[ -f $_dhcp_pidf ] && _dhcpc_pid=$(cat $_dhcp_pidf)
		_dhcpc_cmd=$(eval echo $DHCP4_CMD)

		if [ "$_dhcpc_pid" ]; then
			respawn_off $_dhcpc_cmd || end $?
			# release lease before stopping the daemon
			kill -s USR2 $_dhcpc_pid
			sleep 1
			kill -s TERM $_dhcpc_pid
			_wait=2000000
			_wait_slice=200000
			while [ -d /proc/$_dhcpc_pid ] && [ $_wait -gt 0 ]; do
				usleep $_wait_slice
				_wait=$(($_wait - $_wait_slice))
			done
			if [ $_wait -gt 0 ]; then
				information "DHCPv4 client on" \
					    "$_dhcp_iface stopped"
			else
				end 1 "Couldn't stop DHCPv4 client on" \
				      "$_dhcp_iface"
			fi
		fi
	done
}

# check if a certain interface is running dhcp
check_dhcp() {
	[ $# -lt 1 ] && end 1 "$0: missing args"
	_dhcp_pidf="$DHCP_RUNDIR/dhcp4_$1.pid"
	[ -e $_dhcp_pidf ] || return 1
	check_pid $(cat $_dhcp_pidf)
}

# got a lease on this iface?
check_dhcp_lease() {
	[ $# -lt 1 ] && end 1 "$0: missing args"
	_dhcp_iface=$1
	check_dhcp $_dhcp_iface && [ -e $DHCP_RUNDIR/dhcp4_$_dhcp_iface ] &&
		[ "$(cat $DHCP_RUNDIR/dhcp4_$_dhcp_iface)" = 1 ]
}

# output list of dhcp configured interfaces
dhcp_ifaces() {
	for _dhcp_iface in $(ip link show | \
			sed -rne 's/^[0-9]+:[[:space:]]+([^:]+):.*$/\1/p')
	do
		check_dhcp_lease $_dhcp_iface && echo -n "$_dhcp_iface "
	done
	return 0
}
